import { Announcement } from '../api/announcement/types'
import { isCategory } from '../assets/category'
import { fallbackToUndefined, isInt } from './types'

const filterNames = ['userId', 'category', 'metro', 'bookedBy'] as const
type FilterNames = typeof filterNames[number]

type FiltersType = Partial<Pick<Announcement, FilterNames>>

const defaultFilters: FiltersType = { userId: undefined, category: undefined, metro: undefined, bookedBy: undefined }

const URLEncodeFilters = (filters: FiltersType) => (
    Object.fromEntries(
        filterNames.map(
            fName => {
                const v = filters[fName]
                if (v) {
                    return [fName, v.toString()]
                }
                return [fName, undefined]
            }
        ).filter((p): p is [string, string] => typeof p[1] !== 'undefined')
    )
)

const URLDecoreFilters = (params: URLSearchParams): FiltersType => {
    const strFilters = Object.fromEntries(
        filterNames.map(
            fName => [fName, params.get(fName)]
        ).filter((p): p is [string, string] => p[1] !== null)
    )

    return {
        bookedBy: fallbackToUndefined(strFilters['bookedBy'], isInt),
        category: fallbackToUndefined(strFilters['category'], isCategory),
        metro: strFilters['metro'],
        userId: fallbackToUndefined(strFilters['userId'], isInt)
    }
}

export type { FilterNames, FiltersType }
export { defaultFilters, filterNames, URLEncodeFilters, URLDecoreFilters }
